import { styled } from "solid-styled-components";
import { Show, createSignal, onMount } from "solid-js";
import { vector2d } from "./vector";

import Player from "./objects/player";
import Crystal from "./objects/crystal";
import TimerProvider from "./providers/timer";
import CooldownProvider from "./providers/cooldown";
import CooldownUI from "./ui/cooldown";

const StageContainer = styled("div")`
  width: 100vw;
  height: 100vh;

  position: relative;

  overflow: hidden;
`;

export default function Stage() {
  let container: HTMLDivElement = document.createElement("div");

  let centerPosition = new vector2d(0, 0);
  let playerPosition = new vector2d(0, 0);

  let [ready, setReady] = createSignal(false);

  function initializeStage(element: HTMLDivElement) {
    centerPosition = new vector2d(
      element.clientWidth / 2,
      element.clientHeight / 2
    );
    playerPosition = centerPosition.clone();

    setReady(true);
  }

  onMount(() => initializeStage(container));

  return (
    <TimerProvider>
      <CooldownProvider>
        <StageContainer ref={container}>
          <Show when={ready()}>
            {/* Objects */}
            <Crystal size={128} position={centerPosition} />
            <Player size={64} position={playerPosition} />

            {/* UI Elements */}
            <CooldownUI />
          </Show>
        </StageContainer>
      </CooldownProvider>
    </TimerProvider>
  );
}
