import { createContext, createSignal, useContext } from "solid-js";
import { TickDuration, useTimer } from "./timer";

const CooldownContext = createContext();

export default function CooldownProvider(props: any) {
  const [cooldown, setCooldown] = createSignal<{ [id: string]: number }>({});

  const store = [
    cooldown,
    {
      freeze(id: string, duration: number) {
        const original = structuredClone(cooldown());
        original[id] = duration;
        setCooldown(original);
      },
      unfreeze(id: string) {
        const original = structuredClone(cooldown());
        original[id] = 0;
        setCooldown(original);
      },
    },
  ];

  function reduceCooldown(amount: number = 1000) {
    let triggered = false;
    const original = structuredClone(cooldown());
    Object.keys(original).forEach((k) => {
      if (original[k] > 0) {
        original[k] -= amount;
        triggered = true;
      }
    });

    if (triggered) setCooldown(original);
  }

  const [{ subscribe }] = useTimer();

  subscribe(() => reduceCooldown(TickDuration));

  return (
    <CooldownContext.Provider value={store}>
      {props.children}
    </CooldownContext.Provider>
  );
}

export function useCooldown(): any[2] {
  return useContext(CooldownContext);
}
